'use strict';

/*
==========================================================================
                            RYSOWNIK
==========================================================================
*/
/*********************************************** dep utils ********* */
var _ = require('ramda');
// http://ramdajs.com/0.19.0/docs/

/*********************************************** vars ************** */
var symbols = {
  dash: '-', //dash
  period: '.', //period
  colon: ':', //colon
  pipe: '|', //pipe
  space: ' ', //space
  newLine: '\n' //newLine
};

/*********************************************** helper fns ********* */
// lineGluer:: [wiersze do wyswietlenia] -> "spoiwo" -> str
var lineGluer = function lineGluer(spoiwo) {
  return _.join(spoiwo);
};

// lineMaker:: [ilosc znakow] -> "symbol" -> str
var lineMaker = function lineMaker(symbol) {
  return _.compose(_.join(''), _.repeat(symbol));
};

// spaceCounter:: int, string -> int
var spaceCounter = function spaceCounter(szerokosc, tytul) {
  var tytulLength = tytul.length;
  var balans = szerokosc >= tytulLength ? Math.floor((szerokosc - tytulLength) / 2 - 1) : 2;
  return balans;
};

/*********************************************** main ********* */
function headerConstructor(szerokosc, tytul) {
  var dashedLine = lineMaker(symbols.dash);
  var spacedLine = lineMaker(symbols.space);

  var marginMiddleLine = spacedLine(spaceCounter(szerokosc, tytul));
  var rawMiddleLine = [marginMiddleLine, tytul, marginMiddleLine];

  var toGlue = lineGluer('');
  var middleLine = toGlue(rawMiddleLine);
  var topLine = dashedLine(szerokosc);
  var downLine = topLine;

  var header = [symbols.newLine, topLine, middleLine, downLine];
  return header;
}

function lineConstructor(symbol) {
  var parsedSymbol = symbol === 'dash' ? symbols.dash : symbol === 'period' ? symbols.period : symbol === 'colon' ? symbols.colon : symbols.dash;
  return lineMaker(parsedSymbol);
}

/**
 * printHeader - Drukuje skonstruowany przy pomocy 'headerConstructor' header.
 * @param  {int} szerokosc szerokość headera
 * @param  {str} tytul     tytuł headera
 * @return {str}           wydruk do console.log
 */
var header = _.curry(function header(szerokosc, tytul) {
  var toGlueNL = lineGluer('\n');
  var rawHeader = headerConstructor(szerokosc, tytul);
  var headerToPrint = toGlueNL(rawHeader);
  return headerToPrint;
});

/**
 * line - Drukuje skonstruowany przy pomocy 'lineConstructor' linie.
 * @param  {int} szerokosc szerokość lini
 * @param  {str} symbol    tytuł lini
 * @return {str}           wydruk do console.log
 */
var line = _.curry(function line(szerokosc, symbol) {
  var makeLineWithSymbol = lineConstructor(symbol);
  var rawLine = makeLineWithSymbol(szerokosc);
  return rawLine;
});

///////////////////////////////////////// production /////////////////


// js: #node module exports
module.exports = {
  header: header, //(szerokosc, tytul)
  line: line };